@@ -33,6 +33,16 @@
 #define ABS(a)   ((a)>=0 ? (a) : -(a))
 #define ARRAY_SIZE(a) (sizeof(a)/sizeof(*(a)))
 
+
+/* Bitfield macros */
+
+/* b is u32 array (or ptr), l is size of it in bits (multiple of 32), p is 0..(l-1) */
+#define BIT32_VAL(p)		(((u32) 1) << ((p) % 32))
+#define BIT32_TEST(b,p)		((b)[(p)/32] & BIT32_VAL(p))
+#define BIT32_SET(b,p)		((b)[(p)/32] |= BIT32_VAL(p))
+#define BIT32_CLR(b,p)		((b)[(p)/32] &= ~BIT32_VAL(p))
+#define BIT32_ZERO(b,l)		memset((b), 0, (l)/8)
+
 #ifndef NULL
 #define NULL ((void *) 0)
 #endif
@@ -68,6 +78,38 @@ typedef s64 btime;
 #endif
 
 
+/* Rate limiting */
+
+struct tbf {
+  bird_clock_t timestamp;		/* Last update */
+  u16 count;				/* Available tokens */
+  u16 burst;				/* Max number of tokens */
+  u16 rate;				/* Rate of replenishment */
+  u16 mark;				/* Whether last op was limited */
+};
+
+/* Default TBF values for rate limiting log messages */
+#define TBF_DEFAULT_LOG_LIMITS { .rate = 1, .burst = 5 }
+
+void tbf_update(struct tbf *f);
+
+static inline int
+tbf_limit(struct tbf *f)
+{
+  tbf_update(f);
+
+  if (!f->count)
+  {
+    f->mark = 1;
+    return 1;
+  }
+
+  f->count--;
+  f->mark = 0;
+  return 0;
+}
+
+
 /* Logging and dying */
 
 typedef struct buffer {
@@ -88,18 +130,12 @@ typedef struct buffer {
 
 #define LOG_BUFFER_SIZE 1024
 
-
-struct rate_limit {
-  bird_clock_t timestamp;
-  int count;
-};
-
 #define log log_msg
 void log_commit(int class, buffer *buf);
-void log_msg(char *msg, ...);
-void log_rl(struct rate_limit *rl, char *msg, ...);
-void die(char *msg, ...) NORET;
-void bug(char *msg, ...) NORET;
+void log_msg(const char *msg, ...);
+void log_rl(struct tbf *rl, const char *msg, ...);
+void die(const char *msg, ...) NORET;
+void bug(const char *msg, ...) NORET;
 
 #define L_DEBUG "\001"			/* Debugging messages */
 #define L_TRACE "\002"			/* Protocol tracing */
@@ -111,7 +147,7 @@ void bug(char *msg, ...) NORET;
 #define L_FATAL "\010"			/* Fatal errors */
 #define L_BUG "\011"			/* BIRD bugs */
 
-void debug(char *msg, ...);		/* Printf to debug output */
+void debug(const char *msg, ...);		/* Printf to debug output */
 
 /* Debugging */
 
