@@ -221,6 +221,16 @@ int bvsnprintf(char *buf, int size, const char *fmt, va_list args)
 			continue;
 
 		case 'm':
+			if (flags & SPECIAL) {
+				if (!errno)
+					continue;
+				if (size < 2)
+					return -1;
+				*str++ = ':';
+				*str++ = ' ';
+				start += 2;
+				size -= 2;
+			}
 			s = strerror(errno);
 			goto str;
 		case 'M':
@@ -273,9 +283,9 @@ int bvsnprintf(char *buf, int size, const char *fmt, va_list args)
 		/* IP address */
 		case 'I':
 			if (flags & SPECIAL)
-				ip_ntox(va_arg(args, ip_addr), ipbuf);
+				ipa_ntox(va_arg(args, ip_addr), ipbuf);
 			else {
-				ip_ntop(va_arg(args, ip_addr), ipbuf);
+				ipa_ntop(va_arg(args, ip_addr), ipbuf);
 				if (field_width == 1)
 					field_width = STD_ADDRESS_P_LENGTH;
 			}
