@@ -23,6 +23,13 @@ struct password_item {
 extern struct password_item *last_password_item;
 
 struct password_item *password_find(list *l, int first_fit);
-void password_cpy(char *dst, char *src, int size);
+struct password_item *password_find_by_id(list *l, int id);
+
+static inline int password_verify(struct password_item *p1, char *p2, uint size)
+{
+  char buf[size];
+  strncpy(buf, p1->password, size);
+  return !memcmp(buf, p2, size);
+}
 
 #endif
