@@ -40,6 +40,7 @@
  * Supported standards:
  * - RFC 4861 - main RA standard
  * - RFC 6106 - DNS extensions (RDDNS, DNSSL)
+ * - RFC 4191 (partial) - Default Router Preference
  */
 
 static void
@@ -206,7 +207,7 @@ radv_iface_remove(struct radv_iface *ifa)
 
 static void
 radv_if_notify(struct proto *p, unsigned flags, struct iface *iface)
-{ 
+{
   struct proto_radv *ra = (struct proto_radv *) p;
   struct radv_config *cf = (struct radv_config *) (p->cf);
 
@@ -360,7 +361,7 @@ radv_reconfigure(struct proto *p, struct proto_config *c)
   // struct radv_config *old = (struct radv_config *) (p->cf);
   struct radv_config *new = (struct radv_config *) c;
 
-  /* 
+  /*
    * The question is why there is a reconfigure function for RAdv if
    * it has almost none internal state so restarting the protocol
    * would probably suffice. One small reason is that restarting the
@@ -425,6 +426,7 @@ radv_get_status(struct proto *p, byte *buf)
 struct protocol proto_radv = {
   .name =		"RAdv",
   .template =		"radv%d",
+  .config_size =	sizeof(struct radv_config),
   .init =		radv_init,
   .start =		radv_start,
   .shutdown =		radv_shutdown,
